/* Made by Monkey Machine */

import com.mkeym.customsec.interfaces.TradingAccount;
import com.mkeym.customsec.interfaces.TradingAccountHome;
import javax.ejb.*;
import javax.naming.InitialContext;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;
import javax.rmi.PortableRemoteObject;
import java.util.Iterator;

/**
 * Simple client program which performs a login and then manipulates the accounts.
 * It takes the usearname and password as the first two arguments and one of the follwing combinations: <ol>
 * <li> init - populates the database </li> <li> clean - remove the data entries </li>
 * <li> buy <account name> <stockname> <qty> - buy qty of the specified shares through the given account.</li>
 * <li> sell <account name> <stockname> <qty> - sell qty of the specified shares through the given account.</li> </ol>
 */
public class Client {
	public static void main(String[] args) throws Exception {
		if (args.length < 3)
			showUsageAndExit();
		String username = args[0];
		char[] password = args[1].toCharArray();
		String command = args[2];
		doLogin(username, password);
		if (command.equals("init")) {
			init();
		} else if (command.equals("clean")) {
			clean();
		} else {
			if (args.length != 6)
				showUsageAndExit();
			// Get the account name to make the transaction through
			String accountName = args[3];
			// Get the name and quantity for the stock purchase
			String share = args[4];
			int qty = Integer.parseInt(args[5]);
			InitialContext context = new InitialContext();
			TradingAccountHome home = (TradingAccountHome)PortableRemoteObject.narrow(context.lookup("TradingAccount"),
			    TradingAccountHome.class);
			TradingAccount acct = home.findByPrimaryKey(accountName);
			Handle handle = acct.getHandle();

			acct = (TradingAccount)handle.getEJBObject();

			try {
				if (command.equals("buy")) {
					acct.buy(share, qty);
				} else {
					acct.sell(share, qty);
				}
				int balance = acct.getBalance();
				System.out.println("'Buy' successful on account " + acct.getAccountId() + ", balance is now " + balance);
			} catch (Exception e) {
				System.out.println("Buy failed: " + e.getMessage());
			}
		}
	}

	static void showUsageAndExit() {
		System.out.println("Usage: java Client <username> <password> [init|clean|(buy|sell <acct_name> <stockname> <qty)]");
		System.exit(0);
	}

	/** performs the login for the client. */
	static void doLogin(String username, char[] password) {
		try {
			CustomsecCallbackHandler handler = new CustomsecCallbackHandler(username, password);
			LoginContext lc = new LoginContext("customsec", handler);
			System.out.println("Created LoginContext");
			lc.login();
			System.out.println("Login completed");
		}
		catch (LoginException le) {
			System.out.println("Login failed");
			le.printStackTrace();
		}
	}

	/** intitialises the database by creating a set of trading accounts for the users in the system. */
	private static void init() {
		try {
			InitialContext context = new InitialContext();
			TradingAccountHome home = (TradingAccountHome)PortableRemoteObject.narrow(context.lookup("TradingAccount"),
			    TradingAccountHome.class);
			home.create("yossarian");
			home.create("milo");
			home.create("orr");
			System.out.println("Created accounts successfully.");
		}
		catch (CreateException ce) {
			System.out.println(ce.getMessage());
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	/** removes the created accounts from the database */
	private static void clean() {
		try {
			InitialContext context = new InitialContext();
			TradingAccountHome home = (TradingAccountHome)PortableRemoteObject.narrow(context.lookup("TradingAccount"),
			    TradingAccountHome.class);

/*            TradingAccount acct = home.findByPrimaryKey("orr");
            System.out.println("Calling remove method on " + acct.getClass());
            acct.remove();
			acct = home.findByPrimaryKey("milo");
            System.out.println("Calling remove method on " + acct.getClass());
            acct.remove();
            System.out.println("Calling remove methods on " + home.getClass());
*/

			home.remove("yossarian");
			home.remove("milo");
			home.remove("orr");
			System.out.println("Deleted accounts successfully.");
		}
		catch (RemoveException re) {
			System.out.println(re.getMessage());
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
}
