/* Made by Monkey Machine */

package com.mkeym.customsec.ejb;

import java.lang.reflect.Method;
//import java.util.Random;
import java.security.Principal;
import java.util.Iterator;
import java.util.Enumeration;
import java.security.acl.Group;
import javax.ejb.EJBContext;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.security.auth.Subject;
import org.apache.log4j.Category;

/** A simple security proxy class for the TradingAccountBean.
* This implementation uses the untyped approach - i.e. it implements the <code>org.jboss.security.SecurityProxy</code> interface
* rather than proviting the same interfaec as the bean.
*/
public class TradingAccountSecurityProxy implements org.jboss.security.SecurityProxy {
	private Category _log = Category.getInstance(getClass().getName());

/* There is only one proxy created per container (i.e. not per bean), so we use a ThreadLocal store the invocation context. */

	private ThreadLocal _ctx = new ThreadLocal();

	/**
	 * @label protects
	 * @supplierCardinality 0..*
	 * @labelDirection forward
	 * @clientCardinality 1
	 */

    /*#TradingAccountBean lnkTradingAccountBean;*/

	public void init(Class beanHome, Class beanRemote, Object securityMgr) throws InstantiationException {
		_log.info("Security Proxy Initialising for " + beanRemote.getName());
	}

	/** Called prior to any method invocation to set the current EJB context. */
	public void setEJBContext(EJBContext ctx) {
		_log.info("setEJBContext " + ctx);
		_ctx.set(ctx);
	}

	/**
	 * Called to allow the security proxy to perform any custom security checks required for the EJB home interface method.
	 * @param m, the EJB home remote interface method that is being invoked
	 */
	public void invokeHome(Method m, Object[] args) throws SecurityException {
		EJBContext ctx = (EJBContext)_ctx.get();
		_log.info("invokeHome " + m.getName() + " on " + m.getDeclaringClass().getName() + " called by " +
		    ctx.getCallerPrincipal().getName());
	}

	/**
	 * Called to allow the security proxy to perform any custom security checks required for the EJB remote interface method.
	 * @param m, the EJB remote interface method that is being invoked
	 * @param bean, the EJB implementation class instance
	 */
	public void invoke(Method m, Object[] args, Object bean) throws SecurityException {
		if (!(bean instanceof TradingAccountBean))
			throw new SecurityException("Invalid bean instance for security proxy");
/* 
		// Uncomment as alternative to using EJBContext: lookup the active subject ...
		try {
			InitialContext ic = new InitialContext();
			Subject subject = (Subject)ic.lookup("java:comp/env/security/subject");
			//			System.out.println("Subject: readonly = " + subject.isReadOnly());
			// To list the Principals contained in the Subject...
			Iterator principals = subject.getPrincipals().iterator();
			while (principals.hasNext()) {
				Principal p = (Principal)principals.next();
				_log.info("Principal (" + p.getClass().getName() + ") : " + p.getName());
			}
			// To get the roles (the instance of java.security.acl.Group in the list of Principals)
			principals = subject.getPrincipals(java.security.acl.Group.class).iterator();
			if (principals.hasNext()) {
				Group roles = (Group)principals.next();
				Enumeration roleEnum = roles.members();
				while (roleEnum.hasMoreElements()) {
					_log.info("Role:  " + roleEnum.nextElement());
				}
			}
		} catch (NamingException ne) {
			_log.error("Lookup on Subject failed ", ne);
			throw new SecurityException("Unable to obtain active Subject");
		}
*/
		EJBContext ctx = (EJBContext)_ctx.get();
		String caller = ctx.getCallerPrincipal().getName();
		TradingAccountBean account = (TradingAccountBean)bean;
		String operation = m.getName();
		_log.info("invoke " + operation + " called by " + caller);

		// The implementation of the defined policies.
		if (operation.equals("buy") || operation.equals("sell")) {
			int qty = ((Integer)args[1]).intValue();
			// 1. A trader can only buy or sell shares through their own account.
			if (!account.getAccountId().equals(caller)) {
				_log.warn("method " + operation + " called on account " + account.getAccountId() +
				    " by invalid user " + caller);
				throw new SecurityException("caller is not the account owner");
			}
			int value = qty * 100; // in our kid-on example all the shares cost 100.
			// 2. A single transaction must have a value less than or equal to 50 million.
			if (value > 5e7) {
				_log.warn(operation + " invoked with too high transaction value: " + value);
				throw new SecurityException("transaction value is too large");
			}
			int newBalance = account.getBalance() + (operation.equals("sell") ? value : -value);
			// 3. The balance of an account cannot exceed 100 million or go below -100 million.
			if (Math.abs(newBalance) >= 1e8) {
				_log.warn(operation + " would exceed bounds on account balance: " + newBalance);
				throw new SecurityException("balance exceeded");
			}
		}
	}
}
